@@ -16,25 +16,22 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-permission-manager.c,v 1.8 2004/06/23 20:07:43 chpe Exp $
+ *  $Id: ephy-permission-manager.c,v 1.14 2006/01/22 12:35:20 crispin Exp $
  */
 
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
 
 #include "ephy-permission-manager.h"
+#include "ephy-embed-type-builtins.h"
 #include "ephy-marshal.h"
 #include "ephy-debug.h"
 
-/* EphyPermissionInfo */
-
 GType
 ephy_permission_info_get_type (void)
 {
 	static GType type = 0;
 
-	if (type == 0)
+	if (G_UNLIKELY (type == 0))
 	{
 		type = g_boxed_type_register_static ("EphyPermissionInfo",
 						     (GBoxedCopyFunc) ephy_permission_info_copy,
@@ -109,7 +106,7 @@ ephy_permission_manager_get_type (void)
 {
        static GType type = 0;
 
-	if (type == 0)
+	if (G_UNLIKELY (type == 0))
 	{
 		static const GTypeInfo our_info =
 		{
@@ -150,7 +147,7 @@ ephy_permission_manager_base_init (gpointer g_class)
 		      g_cclosure_marshal_VOID__BOXED,
 		      G_TYPE_NONE,
 		      1,
-		      EPHY_TYPE_PERMISSION_INFO);
+		      EPHY_TYPE_PERMISSION_INFO | G_SIGNAL_TYPE_STATIC_SCOPE);
 
 	/**
 	 * EphyPermissionManager::permission-changed
@@ -168,7 +165,7 @@ ephy_permission_manager_base_init (gpointer g_class)
 		      g_cclosure_marshal_VOID__BOXED,
 		      G_TYPE_NONE,
 		      1,
-		      EPHY_TYPE_PERMISSION_INFO);
+		      EPHY_TYPE_PERMISSION_INFO | G_SIGNAL_TYPE_STATIC_SCOPE);
 
 	/**
 	 * EphyPermissionManager::permission-deleted
@@ -186,7 +183,7 @@ ephy_permission_manager_base_init (gpointer g_class)
 		      g_cclosure_marshal_VOID__BOXED,
 		      G_TYPE_NONE,
 		      1,
-		      EPHY_TYPE_PERMISSION_INFO);
+		      EPHY_TYPE_PERMISSION_INFO | G_SIGNAL_TYPE_STATIC_SCOPE);
 
 	/**
 	 * EphyPermissionManager::permissions-cleared
@@ -209,7 +206,7 @@ ephy_permission_manager_base_init (gpointer g_class)
 }
 
 /**
- * ephy_permission_manager_add:
+ * ephy_permission_manager_add_permission:
  * @manager: the #EphyPermissionManager
  * @host: a website URL
  * @type: a string to identify the type of the permission
@@ -218,17 +215,17 @@ ephy_permission_manager_base_init (gpointer g_class)
  * Adds the specified permission to the permissions database.
  **/
 void
-ephy_permission_manager_add (EphyPermissionManager *manager,
-			     const char *host,
-			     const char *type,
-			     EphyPermission permission)
+ephy_permission_manager_add_permission (EphyPermissionManager *manager,
+					const char *host,
+					const char *type,
+					EphyPermission permission)
 {
 	EphyPermissionManagerIface *iface = EPHY_PERMISSION_MANAGER_GET_IFACE (manager);
 	iface->add (manager, host, type, permission);
 }
 
 /**
- * ephy_permission_manager_remove:
+ * ephy_permission_manager_remove_permission:
  * @manager: the #EphyPermissionManager
  * @host: a website URL
  * @type: a string to identify the type of the permission
@@ -238,29 +235,29 @@ ephy_permission_manager_add (EphyPermissionManager *manager,
  * @host's web pages.
  **/
 void
-ephy_permission_manager_remove (EphyPermissionManager *manager,
-				const char *host,
-				const char *type)
+ephy_permission_manager_remove_permission (EphyPermissionManager *manager,
+					   const char *host,
+					   const char *type)
 {
 	EphyPermissionManagerIface *iface = EPHY_PERMISSION_MANAGER_GET_IFACE (manager);
 	iface->remove (manager, host, type);
 }
 
 /**
- * ephy_permission_manager_clear:
+ * ephy_permission_manager_clear_permission:
  * @manager: the #EphyPermissionManager
  * 
  * Clears the permissions database. This cannot be undone.
  **/
 void
-ephy_permission_manager_clear (EphyPermissionManager *manager)
+ephy_permission_manager_clear_permissions (EphyPermissionManager *manager)
 {
 	EphyPermissionManagerIface *iface = EPHY_PERMISSION_MANAGER_GET_IFACE (manager);
 	iface->clear (manager);
 }
 
 /**
- * ephy_permission_manager_test:
+ * ephy_permission_manager_test_permission:
  * @manager: the #EphyPermissionManager
  * @host: a website URL
  * @type: a string to identify the type of the permission
@@ -273,16 +270,16 @@ ephy_permission_manager_clear (EphyPermissionManager *manager)
  * Return value: the permission of type #EphyPermission
  **/
 EphyPermission
-ephy_permission_manager_test (EphyPermissionManager *manager,
-			      const char *host,
-			      const char *type)
+ephy_permission_manager_test_permission (EphyPermissionManager *manager,
+					 const char *host,
+					 const char *type)
 {
 	EphyPermissionManagerIface *iface = EPHY_PERMISSION_MANAGER_GET_IFACE (manager);
 	return iface->test (manager, host, type);
 }
 
 /**
- * ephy_permission_manager_list:
+ * ephy_permission_manager_list_permission:
  * @manager: the #EphyPermissionManager
  * @type: a string to identify the type of the permission
  * 
@@ -293,8 +290,8 @@ ephy_permission_manager_test (EphyPermissionManager *manager,
  * Return value: the list of permission database entries
  **/
 GList *
-ephy_permission_manager_list (EphyPermissionManager *manager,
-			      const char *type)
+ephy_permission_manager_list_permissions (EphyPermissionManager *manager,
+					 const char *type)
 {
 	EphyPermissionManagerIface *iface = EPHY_PERMISSION_MANAGER_GET_IFACE (manager);
 	return iface->list (manager, type);
