@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: EphyContentPolicy.cpp,v 1.19 2006/01/08 22:28:09 jframeau Exp $
+ *  $Id: EphyContentPolicy.cpp,v 1.20 2006/01/18 15:53:54 chpe Exp $
  */
 
 #include "mozilla-config.h"
@@ -55,9 +55,6 @@ EphyContentPolicy::EphyContentPolicy()
 	mLocked = eel_gconf_get_boolean (CONF_LOCKDOWN_DISABLE_UNSAFE_PROTOCOLS);
 
 	mSafeProtocols = eel_gconf_get_string_list (CONF_LOCKDOWN_ADDITIONAL_SAFE_PROTOCOLS);
-
-	mEmbedSingle = ephy_embed_shell_get_embed_single (embed_shell);
-	g_return_if_fail (mEmbedSingle);
 }
 
 EphyContentPolicy::~EphyContentPolicy()
