@@ -38,7 +38,7 @@
  *
  *  Copyright (C) 2005 Christian Persch
  *
- *  $Id: GtkNSSSecurityWarningDialogs.cpp,v 1.7.4.1 2006/02/02 22:28:04 chpe Exp $
+ *  $Id: GtkNSSSecurityWarningDialogs.cpp,v 1.11 2006/02/02 22:25:18 chpe Exp $
  */
 
 #include "mozilla-config.h"
@@ -92,8 +92,7 @@ GtkNSSSecurityWarningDialogs::ConfirmEnteringSecure (nsIInterfaceRequestor *aCon
 		  GTK_RESPONSE_OK,
 		  _("Security Notice"),
 		  _("This page is loaded over a secure connection"),
-		  _("You can always see the security status of a page from "
-		    "the padlock icon on the statusbar."),
+		  _("The padlock icon in the statusbar indicates whether a page is secure."),
 		  nsnull, _retval);
 
 	*_retval = PR_TRUE;
@@ -271,6 +270,7 @@ GtkNSSSecurityWarningDialogs::DoDialog (nsIInterfaceRequestor *aContext,
 	gtk_window_set_icon_name (GTK_WINDOW (dialog), "web-browser");
 
 	int response = gtk_dialog_run (GTK_DIALOG (dialog));
+	gtk_widget_destroy (dialog);
 
 	*_retval = (response == GTK_RESPONSE_ACCEPT || response == GTK_RESPONSE_OK);
 
@@ -279,6 +279,5 @@ GtkNSSSecurityWarningDialogs::DoDialog (nsIInterfaceRequestor *aContext,
 		prefBranch->SetBoolPref (showOncePref, PR_FALSE);
 	}
 
-	gtk_widget_destroy (dialog);
 	g_free (showOncePref);
 }
