@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: MozRegisterComponents.cpp,v 1.43 2006/01/12 23:21:14 chpe Exp $
+ *  $Id: MozRegisterComponents.cpp,v 1.44 2006/02/02 17:56:28 chpe Exp $
  */
 
 #include "mozilla-config.h"
@@ -229,6 +229,14 @@ static const nsModuleComponentInfo sAppComps[] = {
 		"@mozilla.org/embedcomp/prompt-service;1",
 		EphyPromptServiceConstructor
 	},
+#ifdef HAVE_NSINONBLOCKINGALERTSERVICE_H
+	{
+		EPHY_PROMPT_SERVICE_CLASSNAME,
+		EPHY_PROMPT_SERVICE_IID,
+		"@mozilla.org/embedcomp/nbalert-service;1",
+		EphyPromptServiceConstructor
+	},
+#endif /* HAVE_NSINONBLOCKINGALERTSERVICE_H */
 };
 
 #if defined(HAVE_MOZILLA_PSM) && !defined(HAVE_GECKO_1_8)
