@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-state.h,v 1.8 2004/10/08 12:18:58 marco Exp $
+ *  $Id: ephy-state.h,v 1.13 2006/02/04 13:57:42 paharvey Exp $
  */
 
 #ifndef EPHY_STATE_H
@@ -34,22 +34,22 @@ typedef enum
 	EPHY_STATE_WINDOW_SAVE_POSITION = 1 << 1
 } EphyStateWindowFlags;
 
-void ephy_state_add_window	(GtkWidget *window,
+void  ephy_state_add_window	(GtkWidget *window,
 				 const char *name,
 				 int default_width,
 				 int default_heigth,
 				 gboolean maximize,
 				 EphyStateWindowFlags flags);
 
-void ephy_state_add_paned	(GtkWidget *paned,
+void  ephy_state_add_paned	(GtkWidget *paned,
 				 const char *name,
 				 int default_width);
 
-void ephy_state_add_expander	(GtkWidget *expander,
+void  ephy_state_add_expander	(GtkWidget *widget,
 				 const char *name,
 				 gboolean default_state);
 
-void ephy_state_save		(void);
+void  ephy_state_save		(void);
 
 G_END_DECLS
 
