@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-stock-icons.h,v 1.14.6.1 2005/10/20 13:13:54 chpe Exp $
+ *  $Id: ephy-stock-icons.h,v 1.18 2006/01/16 22:25:16 paharvey Exp $
  */
 
 #ifndef EPHY_STOCK_ICONS_H
@@ -23,9 +23,6 @@
 
 G_BEGIN_DECLS
 
-#define EPHY_ICON_SIZE_TAB_BUTTON "epiphany-icon-size-tab-button"
-
-#define EPHY_STOCK_CLOSE_TAB	   "epiphany-close-tab"
 #define EPHY_STOCK_POPUPS	   "epiphany-popup-hidden"
 #define EPHY_STOCK_HISTORY         "epiphany-history"
 #define EPHY_STOCK_BOOKMARKS       "epiphany-bookmarks"
@@ -36,6 +33,7 @@ G_BEGIN_DECLS
 #define STOCK_VIEW_SOURCE	   "stock_view-html-source"
 #define STOCK_SEND_MAIL		   "stock_mail-send"
 #define STOCK_ADD_BOOKMARK	   "stock_add-bookmark"
+#define STOCK_BOOKMARK		   "stock_bookmark"
 #define STOCK_PRINT_SETUP	   "stock_print-setup"
 #define STOCK_LOCK_INSECURE	   "stock_lock-open"
 #define STOCK_LOCK_SECURE	   "stock_lock"
