@@ -15,12 +15,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-completion-model.c,v 1.15 2004/10/23 16:49:10 chpe Exp $
+ *  $Id: ephy-completion-model.c,v 1.17 2005/01/08 19:41:47 chpe Exp $
  */
 
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
 
 #include "ephy-completion-model.h"
 #include "ephy-node.h"
@@ -33,7 +31,7 @@ static void ephy_completion_model_tree_model_init (GtkTreeModelIface *iface);
 
 #define EPHY_COMPLETION_MODEL_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_COMPLETION_MODEL, EphyCompletionModelPrivate))
 
-struct EphyCompletionModelPrivate
+struct _EphyCompletionModelPrivate
 {
 	EphyNode *history;
 	EphyNode *bookmarks;
