@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-extensions-manager.h,v 1.8 2005/10/21 15:31:19 chpe Exp $
+ *  $Id: ephy-extensions-manager.h,v 1.9 2006/02/07 23:02:35 crispin Exp $
  */
 
 #ifndef EPHY_EXTENSIONS_MANAGER_H
@@ -45,7 +45,8 @@ typedef struct
 {
 	char *identifier;
 	GKeyFile *keyfile;
-	gboolean active;
+	guint active  :1;
+	guint enabled :1;
 } EphyExtensionInfo;
 	
 struct _EphyExtensionsManagerClass
