@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-link.c,v 1.6 2005/02/24 15:00:17 chpe Exp $
+ *  $Id: ephy-link.c,v 1.7 2006/01/29 21:47:21 chpe Exp $
  */
 
 #include "config.h"
@@ -25,6 +25,7 @@
 #include "ephy-type-builtins.h"
 #include "ephy-marshal.h"
 #include "ephy-signal-accumulator.h"
+#include "ephy-gui.h"
 #include "ephy-debug.h"
 
 enum
@@ -97,3 +98,38 @@ ephy_link_open (EphyLink *link,
 
 	return new_tab;
 }
+
+EphyLinkFlags
+ephy_link_flags_from_current_event (void)
+{
+	GdkEventType type = GDK_NOTHING;
+	guint state = 0, button = (guint) -1;
+	EphyLinkFlags flags = 0;
+
+	ephy_gui_get_current_event (&type, &state, &button);
+
+	if (button == 2 && type == GDK_BUTTON_PRESS)
+	{
+		if (state == GDK_SHIFT_MASK)
+		{
+			flags = EPHY_LINK_NEW_WINDOW;
+		}
+		else
+		{
+			flags = EPHY_LINK_NEW_TAB;
+		}
+	}
+	else
+	{
+		if (state == (GDK_CONTROL_MASK | GDK_SHIFT_MASK))
+		{
+			flags = EPHY_LINK_NEW_WINDOW;
+		}
+		else if (state == GDK_CONTROL_MASK)
+		{
+			flags = EPHY_LINK_NEW_TAB;
+		}
+	}
+
+	return flags;
+}
