@@ -18,7 +18,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-tab.c,v 1.171 2006/01/29 21:47:21 chpe Exp $
+ *  $Id: ephy-tab.c,v 1.172 2006/02/08 22:20:02 jframeau Exp $
  */
 
 #include "config.h"
@@ -535,7 +535,7 @@ popups_manager_add (EphyTab *tab,
 
 	popup = g_new0 (PopupInfo, 1);
 
-	popup->url = g_strdup (url);
+	popup->url = (url == NULL) ? NULL : g_strdup (url);
 	popup->name = g_strdup (name);
 	popup->features = g_strdup (features);
 
@@ -651,14 +651,17 @@ popups_manager_show (PopupInfo *popup,
 	EphyEmbed *embed;
 	EphyEmbedSingle *single;
 
-	embed = ephy_tab_get_embed (tab);
+	/* Only show popup with non NULL url */
+	if (popup->url != NULL)
+	{
+		embed = ephy_tab_get_embed (tab);
 
-	single = EPHY_EMBED_SINGLE
-		(ephy_embed_shell_get_embed_single (embed_shell));
-
-	ephy_embed_single_open_window (single, embed, popup->url,
-				       popup->name, popup->features);
+		single = EPHY_EMBED_SINGLE
+			(ephy_embed_shell_get_embed_single (embed_shell));
 
+		ephy_embed_single_open_window (single, embed, popup->url,
+				popup->name, popup->features);
+	}
 	popups_manager_free_info (popup);
 }
 
