@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: prefs-dialog.c,v 1.115 2005/10/15 14:50:00 chpe Exp $
+ *  $Id: prefs-dialog.c,v 1.121 2006/01/17 19:44:03 chpe Exp $
  */
 
 #include "config.h"
@@ -58,35 +58,17 @@
 #include <gtk/gtkstock.h>
 #include <gtk/gtknotebook.h>
 #include <gtk/gtkfilechooserbutton.h>
+#include <gtk/gtkmain.h>
 #include <string.h>
 
+#include <libgnomevfs/gnome-vfs-utils.h>
+
 #define CONF_FONTS_FOR_LANGUAGE	"/apps/epiphany/dialogs/preferences_font_language"
 #define DOWNLOAD_BUTTON_WIDTH	8
 
 static void prefs_dialog_class_init	(PrefsDialogClass *klass);
 static void prefs_dialog_init		(PrefsDialog *pd);
 
-/* Glade callbacks */
-void prefs_clear_cache_button_clicked_cb	(GtkWidget *button,
-						 gpointer data);
-void prefs_dialog_response_cb			(GtkDialog *widget,
-						 gint response_id,
-						 EphyDialog *dialog);
-void prefs_homepage_current_button_clicked_cb	(GtkWidget *button,
-						 EphyDialog *dialog);
-void prefs_homepage_blank_button_clicked_cb	(GtkWidget *button,
-						 EphyDialog *dialog);
-void prefs_language_more_button_clicked_cb	(GtkWidget *button,
-						 EphyDialog *dialog);
-void language_editor_add_button_clicked_cb	(GtkWidget *button,
-						 PrefsDialog *pd);
-void language_editor_remove_button_clicked_cb	(GtkWidget *button,
-						 PrefsDialog *pd);
-void language_editor_up_button_clicked_cb	(GtkWidget *button,
-						 PrefsDialog *pd);
-void language_editor_down_button_clicked_cb	(GtkWidget *button,
-						 PrefsDialog *pd);
-
 #include "languages.h"
 
 static const char * const cookies_accept_enum [] =
@@ -141,14 +123,16 @@ enum
 	DOWNLOAD_PATH_HBOX_PROP,
 
 	/* Fonts and Colors */
-	FONTS_LANGUAGE_PROP,
-	VARIABLE_PROP,
-	MONOSPACE_PROP,
-	FIXED_SIZE_PROP,
-	VARIABLE_SIZE_PROP,
-	MIN_SIZE_PROP,
-	USE_COLORS_PROP,
 	USE_FONTS_PROP,
+	MIN_SIZE_PROP,
+	FONT_PREFS_BUTTON_PROP,
+	USE_COLORS_PROP,
+#ifdef HAVE_GECKO_1_8
+	CSS_CONTAINER_PROP,
+	CSS_CHECKBOX_PROP,
+	CSS_EDIT_BOX_PROP,
+	CSS_EDIT_BUTTON_PROP,
+#endif
 
 	/* Privacy */
 	ALLOW_POPUPS_PROP,
@@ -156,6 +140,7 @@ enum
 	ALLOW_JS_PROP,
 	ACCEPT_COOKIES_PROP,
 	DISK_CACHE_PROP,
+	CLEAR_CACHE_BUTTON_PROP,
 
 	/* Language */
 	AUTO_ENCODING_PROP,
@@ -183,14 +168,16 @@ EphyDialogProperty properties [] =
 	{ "download_button_hbox",		NULL,			  PT_NORMAL,	0 },
 
 	/* Fonts and Colors */
-	{ "fonts_language_combo",	CONF_FONTS_FOR_LANGUAGE,	PT_AUTOAPPLY,	G_TYPE_STRING },
-	{ "variable_combo",		NULL,				PT_AUTOAPPLY,	G_TYPE_STRING },
-	{ "monospace_combo",		NULL,				PT_AUTOAPPLY,	G_TYPE_STRING },
-	{ "fixed_size_spinbutton",	NULL,				PT_AUTOAPPLY,	0 },
-	{ "variable_size_spinbutton",	NULL,				PT_AUTOAPPLY,	0 },
-	{ "min_size_spinbutton",	NULL,				PT_AUTOAPPLY,	0 },
-	{ "use_colors_checkbutton",	CONF_RENDERING_USE_OWN_COLORS,	PT_AUTOAPPLY,	0 },
-	{ "use_fonts_checkbutton",	CONF_RENDERING_USE_OWN_FONTS,	PT_AUTOAPPLY,	0 },
+	{ "use_fonts_checkbutton",	CONF_RENDERING_USE_OWN_FONTS,	PT_AUTOAPPLY | PT_INVERTED,	0 },
+	{ "min_size_spinbutton",	CONF_RENDERING_FONT_MIN_SIZE,	PT_AUTOAPPLY,	0 },
+	{ "font_prefs_button",		NULL,				PT_NORMAL,	0 },
+	{ "use_colors_checkbutton",	CONF_RENDERING_USE_OWN_COLORS,	PT_AUTOAPPLY | PT_INVERTED,	0 },
+#ifdef HAVE_GECKO_1_8
+	{ "css_container",		NULL,				PT_NORMAL,	0 },
+	{ "css_checkbox",		CONF_USER_CSS_ENABLED,		PT_AUTOAPPLY,	0 },
+	{ "css_edit_box",		NULL,				PT_NORMAL,	0 },
+	{ "css_edit_button",		NULL,				PT_NORMAL,	0 },
+#endif
 
 	/* Privacy */
 	{ "popups_allow_checkbutton",		CONF_SECURITY_ALLOW_POPUPS,	  PT_AUTOAPPLY, 0 },
@@ -198,6 +185,7 @@ EphyDialogProperty properties [] =
 	{ "enable_javascript_checkbutton",	CONF_SECURITY_JAVASCRIPT_ENABLED, PT_AUTOAPPLY, 0 },
 	{ "cookies_radiobutton",		CONF_SECURITY_COOKIES_ACCEPT,	  PT_AUTOAPPLY, G_TYPE_STRING },
 	{ "disk_cache_spin",			CONF_NETWORK_CACHE_SIZE,	  PT_AUTOAPPLY, 0 },
+	{ "clear_cache_button",			NULL,				  PT_NORMAL,	0 },
 
 	/* Languages */
 	{ "auto_encoding_combo",	CONF_LANGUAGE_AUTODETECT_ENCODING,	PT_AUTOAPPLY,	G_TYPE_STRING },
@@ -230,12 +218,44 @@ EphyDialogProperty add_lang_props [] =
 	{ NULL }
 };
 
+enum
+{
+	FONTS_WINDOW_PROP,
+	FONTS_LANGUAGE_PROP,
+	FONTS_VARIABLE_PROP,
+	FONTS_MONOSPACE_PROP,
+	FONTS_VARIABLE_SIZE_PROP,
+	FONTS_FIXED_SIZE_PROP,
+	FONTS_MIN_SIZE_PROP,
+};
+
+static const
+EphyDialogProperty fonts_properties [] =
+{
+	{ "fonts_dialog",		NULL,				PT_NORMAL, 0},
+	{ "fonts_language_combo",	CONF_FONTS_FOR_LANGUAGE,	PT_AUTOAPPLY,	G_TYPE_STRING },
+	{ "variable_combo",		NULL,				PT_AUTOAPPLY,	G_TYPE_STRING },
+	{ "monospace_combo",		NULL,				PT_AUTOAPPLY,	G_TYPE_STRING },
+	{ "variable_size_spinbutton",	NULL,				PT_AUTOAPPLY,	0 },
+	{ "fixed_size_spinbutton",	NULL,				PT_AUTOAPPLY,	0 },
+	{ "min_size_spinbutton",	NULL,				PT_AUTOAPPLY,	0 },
+
+	{ NULL }
+};
+
 enum
 {
 	COL_FONTS_LANG_NAME,
 	COL_FONTS_LANG_CODE
 };
 
+enum
+{
+	COL_FONT_NAME,
+	COL_FONT_DATA,
+	COL_FONT_IS_SEP
+};
+
 enum
 {
 	COL_LANG_NAME,
@@ -248,12 +268,20 @@ enum
 	COL_ENC_CODE
 };
 
+enum
+{
+	COL_AUTODETECTOR_NAME,
+	COL_AUTODETECTOR_DATA,
+	COL_AUTODETECTOR_IS_SEP
+};
+
 #define EPHY_PREFS_DIALOG_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_PREFS_DIALOG, PrefsDialogPrivate))
 
 struct PrefsDialogPrivate
 {
 	GtkTreeView *lang_treeview;
 	GtkTreeModel *lang_model;
+	EphyDialog *fonts_dialog;
 	EphyDialog *add_lang_dialog;
 	GtkWidget *lang_add_button;
 	GtkWidget *lang_remove_button;
@@ -297,17 +325,26 @@ static void
 prefs_dialog_finalize (GObject *object)
 {
 	PrefsDialog *dialog = EPHY_PREFS_DIALOG (object);
+	PrefsDialogPrivate *priv = dialog->priv;
 
-	if (dialog->priv->add_lang_dialog != NULL)
+	if (priv->fonts_dialog != NULL)
 	{
 		g_object_remove_weak_pointer
-			(G_OBJECT (dialog->priv->add_lang_dialog),
-			(gpointer *) &dialog->priv->add_lang_dialog);
-		g_object_unref (dialog->priv->add_lang_dialog);
+			(G_OBJECT (priv->fonts_dialog),
+			(gpointer *) &priv->fonts_dialog);
+		g_object_unref (priv->fonts_dialog);
 	}
 
-	g_hash_table_destroy (dialog->priv->iso_639_table);
-	g_hash_table_destroy (dialog->priv->iso_3166_table);
+	if (priv->add_lang_dialog != NULL)
+	{
+		g_object_remove_weak_pointer
+				(G_OBJECT (priv->add_lang_dialog),
+				 (gpointer *) &priv->add_lang_dialog);
+		g_object_unref (priv->add_lang_dialog);
+	}
+
+	g_hash_table_destroy (priv->iso_639_table);
+	g_hash_table_destroy (priv->iso_3166_table);
 
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -356,58 +393,53 @@ setup_font_combo (EphyDialog *dialog,
 {
 	GtkWidget *combo;
 	GtkListStore *store;
-	GtkTreeModel *sortmodel;
 	GtkTreeIter iter;
 	GList *fonts, *l;
-	char *name;
 	char key[255];
 	EphyEmbedSingle *single;
 
 	single = EPHY_EMBED_SINGLE (ephy_embed_shell_get_embed_single (embed_shell));
 	fonts = ephy_embed_single_get_font_list (single, code);
+	fonts = g_list_sort (fonts, (GCompareFunc) strcmp);
 
 	g_snprintf (key, 255, "%s_%s_%s", CONF_RENDERING_FONT, type, code);
-	name = eel_gconf_get_string (key);
 
-	/* sanitise the pref */
-	if (name == NULL || name[0] == '\0'
-	    || g_list_find_custom (fonts, name, (GCompareFunc) strcmp) == NULL)
-	{
-		if (prop == VARIABLE_PROP)
-		{
-			eel_gconf_set_string (key, "sans-serif");
-		}
-		else
-		{
-			eel_gconf_set_string (key, "monospace");
-		}
-	}
-	g_free (name);
+	combo = ephy_dialog_get_control (dialog, fonts_properties[prop].id);
+	store = gtk_list_store_new (3, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_BOOLEAN);
 
-	combo = ephy_dialog_get_control (dialog, properties[prop].id);
-	store = gtk_list_store_new (1, G_TYPE_STRING);
+	gtk_list_store_append (store, &iter);
+	gtk_list_store_set (store, &iter,
+			    COL_FONT_NAME, _("Default"),
+			    COL_FONT_DATA, NULL,
+			    COL_FONT_IS_SEP, FALSE,
+			    -1);
+	gtk_list_store_append (store, &iter);
+	gtk_list_store_set (store, &iter,
+			    COL_FONT_NAME, "",
+			    COL_FONT_DATA, "",
+			    COL_FONT_IS_SEP, TRUE,
+			    -1);
 
 	for (l = fonts; l != NULL; l = l->next)
 	{
 		gtk_list_store_append (store, &iter);
-		gtk_list_store_set (store, &iter, 0, (char *) l->data, -1);
+		gtk_list_store_set (store, &iter,
+				    COL_FONT_NAME, (char *) l->data,
+				    COL_FONT_DATA, (char *) l->data,
+				    COL_FONT_IS_SEP, FALSE,
+				    -1);
 	}
 	g_list_foreach (fonts, (GFunc) g_free, NULL);
 	g_list_free (fonts);
 
-	sortmodel = gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (store));
-	gtk_tree_sortable_set_sort_column_id
-		(GTK_TREE_SORTABLE (sortmodel), 0, GTK_SORT_ASCENDING);
+	ephy_dialog_set_pref (dialog, fonts_properties[prop].id, NULL);
 
-	ephy_dialog_set_pref (dialog, properties[prop].id, NULL);
-
-	gtk_combo_box_set_model (GTK_COMBO_BOX (combo), sortmodel);
+	gtk_combo_box_set_model (GTK_COMBO_BOX (combo), GTK_TREE_MODEL (store));
 	gtk_combo_box_set_active (GTK_COMBO_BOX (combo), -1);
 
-	ephy_dialog_set_pref (dialog, properties[prop].id, key);
+	ephy_dialog_set_pref (dialog, fonts_properties[prop].id, key);
 
 	g_object_unref (store);
-	g_object_unref (sortmodel);
 }
 
 static void
@@ -423,8 +455,8 @@ fonts_language_changed_cb (EphyDialog *dialog,
 
 	LOG ("fonts language combo changed, new code '%s'", code);
 
-	setup_font_combo (dialog, "variable", code, VARIABLE_PROP);
-	setup_font_combo (dialog, "monospace", code, MONOSPACE_PROP);
+	setup_font_combo (dialog, "variable", code, FONTS_VARIABLE_PROP);
+	setup_font_combo (dialog, "monospace", code, FONTS_MONOSPACE_PROP);
 
 	g_snprintf (key, sizeof (key), "%s_%s", size_prefs[FONT_SIZE_VAR], code);
 	size = eel_gconf_get_integer (key);
@@ -432,7 +464,7 @@ fonts_language_changed_cb (EphyDialog *dialog,
 	{
 		eel_gconf_set_integer (key, default_size[FONT_SIZE_VAR]);
 	}
-	ephy_dialog_set_pref (dialog, properties[VARIABLE_SIZE_PROP].id, key);
+	ephy_dialog_set_pref (dialog, fonts_properties[FONTS_VARIABLE_SIZE_PROP].id, key);
 
 	g_snprintf (key, sizeof (key), "%s_%s", size_prefs[FONT_SIZE_FIXED], code);
 	size = eel_gconf_get_integer (key);
@@ -440,7 +472,7 @@ fonts_language_changed_cb (EphyDialog *dialog,
 	{
 		eel_gconf_set_integer (key, default_size[FONT_SIZE_FIXED]);
 	}
-	ephy_dialog_set_pref (dialog, properties[FIXED_SIZE_PROP].id, key);
+	ephy_dialog_set_pref (dialog, fonts_properties[FONTS_FIXED_SIZE_PROP].id, key);
 
 	g_snprintf (key, sizeof (key), "%s_%s", size_prefs[FONT_SIZE_MIN], code);
 	size = eel_gconf_get_integer (key);
@@ -448,12 +480,41 @@ fonts_language_changed_cb (EphyDialog *dialog,
 	{
 		eel_gconf_set_integer (key, default_size[FONT_SIZE_MIN]);
 	}
-	ephy_dialog_set_pref (dialog, properties[MIN_SIZE_PROP].id, key);
+	ephy_dialog_set_pref (dialog, fonts_properties[FONTS_MIN_SIZE_PROP].id, key);
 }
 
 static void
-create_fonts_language_menu (EphyDialog *dialog)
+font_prefs_dialog_response_cb (GtkWidget *widget,
+			       int response,
+			       EphyDialog *dialog)
 {
+	if (response == GTK_RESPONSE_HELP)
+	{
+		/* FIXME */
+		return;
+	}
+
+	g_object_unref (dialog);
+}
+
+static gboolean
+row_is_separator (GtkTreeModel *model,
+                  GtkTreeIter *iter,
+                  gpointer data)
+{
+	int column = GPOINTER_TO_INT (data);
+	gboolean is_sep;
+
+	gtk_tree_model_get (model, iter, column, &is_sep, -1);
+
+	return is_sep;
+}
+
+static EphyDialog *
+setup_fonts_dialog (PrefsDialog *pd)
+{
+	EphyDialog *dialog;
+	GtkWidget *window, *parent, *variable_combo, *monospace_combo;
 	GtkWidget *combo;
 	GtkCellRenderer *renderer;
 	GtkListStore *store;
@@ -462,7 +523,51 @@ create_fonts_language_menu (EphyDialog *dialog)
 	guint n_fonts_languages, i;
 	const EphyFontsLanguageInfo *fonts_languages;
 
-	combo = ephy_dialog_get_control (dialog, properties[FONTS_LANGUAGE_PROP].id);
+	parent = ephy_dialog_get_control (EPHY_DIALOG (pd),
+					  properties[WINDOW_PROP].id);
+
+	dialog =  EPHY_DIALOG (g_object_new (EPHY_TYPE_DIALOG,
+					     "parent-window", parent,
+					     NULL));
+
+	ephy_dialog_construct (dialog, fonts_properties,
+			       ephy_file ("prefs-dialog.glade"),
+			       "fonts_dialog", NULL);
+
+	ephy_dialog_get_controls
+		(dialog,
+		 fonts_properties[FONTS_WINDOW_PROP].id, &window,
+		 fonts_properties[FONTS_LANGUAGE_PROP].id, &combo,
+		 fonts_properties[FONTS_VARIABLE_PROP].id, &variable_combo,
+		 fonts_properties[FONTS_MONOSPACE_PROP].id, &monospace_combo,
+		 NULL);
+
+	gtk_window_group_add_window (ephy_gui_ensure_window_group (GTK_WINDOW (parent)),
+				     GTK_WINDOW (window));
+	g_signal_connect (window, "response",
+			  G_CALLBACK (font_prefs_dialog_response_cb), dialog);
+
+	renderer = gtk_cell_renderer_text_new ();
+        gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (variable_combo), renderer, TRUE);
+        gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (variable_combo), renderer,
+                                        "text", COL_FONT_NAME,
+                                        NULL);
+	ephy_dialog_set_data_column (dialog, fonts_properties[FONTS_VARIABLE_PROP].id,
+				     COL_FONT_DATA);
+        renderer = gtk_cell_renderer_text_new ();
+        gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (monospace_combo), renderer, TRUE);
+        gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (monospace_combo), renderer,
+                                        "text", COL_FONT_NAME,
+                                        NULL);
+	ephy_dialog_set_data_column (dialog, fonts_properties[FONTS_MONOSPACE_PROP].id,
+				     COL_FONT_DATA);
+
+	gtk_combo_box_set_row_separator_func (GTK_COMBO_BOX (variable_combo),
+					      (GtkTreeViewRowSeparatorFunc) row_is_separator,
+					      GINT_TO_POINTER (COL_FONT_IS_SEP), NULL);
+	gtk_combo_box_set_row_separator_func (GTK_COMBO_BOX (monospace_combo),
+					      (GtkTreeViewRowSeparatorFunc) row_is_separator,
+					      GINT_TO_POINTER (COL_FONT_IS_SEP), NULL);
 
 	store = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_STRING);
 
@@ -479,8 +584,9 @@ create_fonts_language_menu (EphyDialog *dialog)
 	}
 
 	sortmodel = gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (store));
-	gtk_tree_sortable_set_sort_column_id
-		(GTK_TREE_SORTABLE (sortmodel), COL_FONTS_LANG_NAME, GTK_SORT_ASCENDING);
+	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (sortmodel),
+					      COL_FONTS_LANG_NAME,
+					      GTK_SORT_ASCENDING);
 
 	gtk_combo_box_set_model (GTK_COMBO_BOX (combo), sortmodel);
 
@@ -490,7 +596,7 @@ create_fonts_language_menu (EphyDialog *dialog)
                                         "text", COL_FONTS_LANG_NAME,
                                         NULL);
 
-	ephy_dialog_set_data_column (dialog, properties[FONTS_LANGUAGE_PROP].id, COL_FONTS_LANG_CODE);
+	ephy_dialog_set_data_column (dialog, fonts_properties[FONTS_LANGUAGE_PROP].id, COL_FONTS_LANG_CODE);
 
 	g_signal_connect (dialog, "changed::fonts_language_combo",
 			  G_CALLBACK (fonts_language_changed_cb),
@@ -498,8 +604,57 @@ create_fonts_language_menu (EphyDialog *dialog)
 
 	g_object_unref (store);
 	g_object_unref (sortmodel);
+
+	return dialog;
 }
 
+static void
+font_prefs_button_clicked_cb (GtkWidget *button,
+			      PrefsDialog *pd)
+{
+	PrefsDialogPrivate *priv = pd->priv;
+
+	if (priv->fonts_dialog == NULL)
+	{
+		priv->fonts_dialog = setup_fonts_dialog (pd);
+
+		g_object_add_weak_pointer
+			(G_OBJECT (priv->fonts_dialog),
+			(gpointer *) &priv->fonts_dialog);
+	}
+
+	ephy_dialog_show (priv->fonts_dialog);
+}
+
+#ifdef HAVE_GECKO_1_8
+
+static void
+css_checkbox_toggled (GtkToggleButton *button,
+		      GtkWidget *widget)
+{
+	g_object_set (widget, "sensitive", gtk_toggle_button_get_active (button), NULL);
+}
+
+static void
+css_edit_button_clicked_cb (GtkWidget *button,
+			    PrefsDialog *pd)
+{
+	char *css_file, *uri;
+
+	css_file = g_build_filename (ephy_dot_dir (),
+				     USER_STYLESHEET_FILENAME,
+				     NULL);
+	uri = gnome_vfs_get_uri_from_local_path (css_file);
+
+	ephy_file_launch_handler ("text/plain", uri,
+				  gtk_get_current_event_time ());
+
+	g_free (css_file);
+	g_free (uri);
+}
+
+#endif /* HAVE_GECKO_1_8 */
+
 static void
 create_node_combo (EphyDialog *dialog,
 		   int prop,
@@ -551,6 +706,96 @@ create_node_combo (EphyDialog *dialog,
 	g_object_unref (sortmodel);
 }
 
+static void
+create_autodetectors_combo (EphyDialog *dialog,
+			    int prop,
+			    EphyEncodings *encodings,
+			    EphyNode *node,
+			    const char *key,
+			    const char *default_value)
+{
+	GtkListStore *store;
+	GtkTreeIter iter;
+	GPtrArray *children;
+	GtkComboBox *combo;
+	GtkCellRenderer *renderer;
+	char *code;
+	const char *off_title = NULL;
+	guint i;
+
+	code = eel_gconf_get_string (key);
+	if (code == NULL || ephy_encodings_get_node (encodings, code, FALSE) == NULL)
+	{
+		/* safe default */
+		eel_gconf_set_string (key, default_value);
+	}
+	g_free (code);
+
+	combo = GTK_COMBO_BOX (ephy_dialog_get_control (dialog, properties[prop].id));
+
+	store = gtk_list_store_new (3, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_BOOLEAN);
+
+	children = ephy_node_get_children (node);
+	for (i = 0; i < children->len; ++i)
+	{
+		EphyNode *kid = g_ptr_array_index (children, i);
+		const char *code, *title;
+
+		code = ephy_node_get_property_string (kid, EPHY_NODE_ENCODING_PROP_ENCODING);
+		g_return_if_fail (code != NULL);
+
+		title = ephy_node_get_property_string (kid, EPHY_NODE_ENCODING_PROP_TITLE_ELIDED);
+		if (code[0] == '\0')
+		{
+			off_title = title;
+		}
+		else
+		{	
+			gtk_list_store_insert_with_values (store, &iter, -1,
+							   COL_AUTODETECTOR_NAME, title,
+							   COL_AUTODETECTOR_DATA, code,
+							   COL_AUTODETECTOR_IS_SEP, FALSE,
+							   -1);
+		}
+	}
+
+	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (store),
+					      COL_AUTODETECTOR_NAME,
+					      GTK_SORT_ASCENDING);
+	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (store),
+					      GTK_TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID,
+					      GTK_SORT_ASCENDING);
+
+	g_assert (off_title != NULL);
+
+	gtk_list_store_insert_with_values (store, &iter, 0,
+					   COL_AUTODETECTOR_NAME, off_title,
+					   COL_AUTODETECTOR_DATA, "",
+					   COL_AUTODETECTOR_IS_SEP, FALSE,
+					   -1);
+	gtk_list_store_insert_with_values (store, &iter, 1,
+					   COL_AUTODETECTOR_NAME, NULL,
+					   COL_AUTODETECTOR_DATA, "",
+					   COL_AUTODETECTOR_IS_SEP, TRUE,
+					   -1);
+
+	gtk_combo_box_set_model (combo, GTK_TREE_MODEL (store));
+	g_object_unref (store);
+
+	gtk_combo_box_set_row_separator_func (GTK_COMBO_BOX (combo),
+					      (GtkTreeViewRowSeparatorFunc) row_is_separator,
+					      GINT_TO_POINTER (COL_AUTODETECTOR_IS_SEP), NULL);
+
+
+	renderer = gtk_cell_renderer_text_new ();
+	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (combo), renderer, TRUE);
+	gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (combo), renderer,
+					"text", COL_AUTODETECTOR_NAME,
+					NULL);
+
+	ephy_dialog_set_data_column (dialog, properties[prop].id, COL_AUTODETECTOR_DATA);
+}
+
 static void
 language_editor_add (PrefsDialog *pd,
 		     const char *code,
@@ -804,7 +1049,7 @@ static EphyDialog *
 setup_add_language_dialog (PrefsDialog *pd)
 {
 	EphyDialog *dialog;
-	GtkWidget *window;
+	GtkWidget *window, *parent;
 	GtkListStore *store;
 	GtkTreeModel *sortmodel;
 	GtkTreeView *treeview;
@@ -814,10 +1059,10 @@ setup_add_language_dialog (PrefsDialog *pd)
 	GtkTreeIter iter;
 	int i;
 
-	window = ephy_dialog_get_control (EPHY_DIALOG (pd), properties[WINDOW_PROP].id);
+	parent = ephy_dialog_get_control (EPHY_DIALOG (pd), properties[WINDOW_PROP].id);
 
 	dialog =  EPHY_DIALOG (g_object_new (EPHY_TYPE_DIALOG,
-					     "parent-window", window,
+					     "parent-window", parent,
 					     "default-width", 260,
 					     "default-height", 230,
 					     NULL));
@@ -856,6 +1101,9 @@ setup_add_language_dialog (PrefsDialog *pd)
 				  add_lang_props[LANGUAGE_DIALOG].id, &window,
 				  NULL);
 
+	gtk_window_group_add_window (GTK_WINDOW (parent)->group, GTK_WINDOW (window));
+	gtk_window_set_modal (GTK_WINDOW (window), TRUE);
+
 	gtk_tree_view_set_reorderable (GTK_TREE_VIEW (treeview), FALSE);
 
 	gtk_tree_view_set_model (treeview, sortmodel);
@@ -889,7 +1137,7 @@ setup_add_language_dialog (PrefsDialog *pd)
 	return dialog;
 }
 
-void
+static void
 language_editor_add_button_clicked_cb (GtkWidget *button,
 				       PrefsDialog *pd)
 {
@@ -905,7 +1153,7 @@ language_editor_add_button_clicked_cb (GtkWidget *button,
 	ephy_dialog_show (pd->priv->add_lang_dialog);
 }
 
-void
+static void
 language_editor_remove_button_clicked_cb (GtkWidget *button,
 					  PrefsDialog *pd)
 {
@@ -924,7 +1172,7 @@ language_editor_remove_button_clicked_cb (GtkWidget *button,
 	language_editor_update_buttons (pd);
 }
 
-void
+static void
 language_editor_up_button_clicked_cb (GtkWidget *button,
 				      PrefsDialog *pd)
 {
@@ -956,7 +1204,7 @@ language_editor_up_button_clicked_cb (GtkWidget *button,
 	language_editor_update_buttons (pd);
 }
 
-void
+static void
 language_editor_down_button_clicked_cb (GtkWidget *button,
 					PrefsDialog *pd)
 {
@@ -1024,6 +1272,15 @@ create_language_section (EphyDialog *dialog)
 		 properties[LANGUAGE_DOWN_BUTTON_PROP].id, &pd->priv->lang_down_button,
 		 NULL);
 
+	g_signal_connect (pd->priv->lang_add_button, "clicked",
+			  G_CALLBACK (language_editor_add_button_clicked_cb), dialog);
+	g_signal_connect (pd->priv->lang_remove_button, "clicked",
+			  G_CALLBACK (language_editor_remove_button_clicked_cb), dialog);
+	g_signal_connect (pd->priv->lang_up_button, "clicked",
+			  G_CALLBACK (language_editor_up_button_clicked_cb), dialog);
+	g_signal_connect (pd->priv->lang_down_button, "clicked",
+			  G_CALLBACK (language_editor_down_button_clicked_cb), dialog);
+
 	/* setup the languages treeview */
 	pd->priv->lang_treeview = treeview;
 
@@ -1117,17 +1374,34 @@ create_language_section (EphyDialog *dialog)
 static void
 download_path_changed_cb (GtkFileChooser *button)
 {
-	char *dir;
+	char *dir, *downloads_dir, *desktop_dir;
 
 	/* FIXME: use _uri variant when we support downloading 
 	 * to gnome-vfs remote locations
 	 */
 	dir = gtk_file_chooser_get_filename (button);
-	if (dir != NULL)
+	if (dir == NULL) return;
+
+	downloads_dir = ephy_file_downloads_dir ();
+	desktop_dir = ephy_file_desktop_dir ();
+	g_return_if_fail (downloads_dir != NULL && desktop_dir != NULL);
+
+	if (strcmp (dir, downloads_dir) == 0)
+	{
+		eel_gconf_set_string (CONF_STATE_DOWNLOAD_DIR, "Downloads");
+	}
+	else if (strcmp (dir, desktop_dir) == 0)
+	{
+		eel_gconf_set_string (CONF_STATE_DOWNLOAD_DIR, "Desktop");
+	}
+	else
 	{
 		eel_gconf_set_path (CONF_STATE_DOWNLOAD_DIR, dir);
-		g_free (dir);
 	}
+
+	g_free (dir);
+	g_free (downloads_dir);
+	g_free (desktop_dir);
 }
 
 static void
@@ -1137,14 +1411,14 @@ create_download_path_button (EphyDialog *dialog)
 	EphyFileChooser *fc;
 	char *dir;
 
-	dir = ephy_file_downloads_dir ();
+	dir = ephy_file_get_downloads_dir ();
 
 	ephy_dialog_get_controls (dialog,
 				  properties[DOWNLOAD_PATH_HBOX_PROP].id, &hbox,
 				  properties[WINDOW_PROP].id, &parent,
 				  NULL);
 
-	fc = ephy_file_chooser_new (_("Select a directory"),
+	fc = ephy_file_chooser_new (_("Select a Directory"),
 				    parent,
 				    GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
 				    NULL, EPHY_FILE_FILTER_NONE);
@@ -1164,92 +1438,24 @@ create_download_path_button (EphyDialog *dialog)
 	gtk_widget_set_sensitive (button, eel_gconf_key_is_writable (CONF_STATE_DOWNLOAD_DIR));
 	g_free (dir);
 }
-	
+
 static void
-prefs_dialog_init (PrefsDialog *pd)
-{
-	EphyDialog *dialog = EPHY_DIALOG (pd);
-	EphyEncodings *encodings;
-	GtkWidget *window, *curr_button, *blank_button;
-	GtkWidget *variable_combo, *monospace_combo;
-	GtkCellRenderer *renderer;
-	gboolean sensitive;
-
-	pd->priv = EPHY_PREFS_DIALOG_GET_PRIVATE (pd);
-
-	ephy_dialog_construct (dialog,
-			       properties,
-			       ephy_file ("prefs-dialog.glade"),
-			       "prefs_dialog",
-			       NULL);
-
-	ephy_dialog_add_enum (dialog, properties[ACCEPT_COOKIES_PROP].id,
-			      G_N_ELEMENTS (cookies_accept_enum), cookies_accept_enum);
-
-	ephy_dialog_get_controls
-		(dialog,
-		 properties[WINDOW_PROP].id, &window,
-		 properties[HOMEPAGE_CURRENT_PROP].id, &curr_button,
-		 properties[HOMEPAGE_BLANK_PROP].id, &blank_button,
-		 properties[VARIABLE_PROP].id, &variable_combo,
-		 properties[MONOSPACE_PROP].id, &monospace_combo,
-		 NULL);
-
-	gtk_window_set_icon_name (GTK_WINDOW (window), GTK_STOCK_PREFERENCES);
-
-	/* set homepage button sensitivity */
-	sensitive = eel_gconf_key_is_writable (CONF_GENERAL_HOMEPAGE);
-	gtk_widget_set_sensitive (curr_button, sensitive);
-	gtk_widget_set_sensitive (blank_button, sensitive);
-
-        renderer = gtk_cell_renderer_text_new ();
-        gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (variable_combo), renderer, TRUE);
-        gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (variable_combo), renderer,
-                                        "text", 0,
-                                        NULL);
-	ephy_dialog_set_data_column (dialog, properties[VARIABLE_PROP].id, 0);
-        renderer = gtk_cell_renderer_text_new ();
-        gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (monospace_combo), renderer, TRUE);
-        gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (monospace_combo), renderer,
-                                        "text", 0,
-                                        NULL);
-	ephy_dialog_set_data_column (dialog, properties[MONOSPACE_PROP].id, 0);
-
-	create_fonts_language_menu (dialog);
-
-	encodings = EPHY_ENCODINGS (ephy_embed_shell_get_encodings
-					(EPHY_EMBED_SHELL (ephy_shell)));
-
-	create_node_combo (dialog, DEFAULT_ENCODING_PROP, encodings,
-			   ephy_encodings_get_all (encodings),
-			   CONF_LANGUAGE_DEFAULT_ENCODING, "ISO-8859-1");
-	create_node_combo (dialog, AUTO_ENCODING_PROP, encodings,
-			   ephy_encodings_get_detectors (encodings),
-			   CONF_LANGUAGE_AUTODETECT_ENCODING, "");
-
-	create_language_section	(dialog);
-
-	create_download_path_button (dialog);
-}
-
-void
 prefs_dialog_response_cb (GtkDialog *widget,
-			  gint response_id,
+			  int response,
 			  EphyDialog *dialog)
 {
-	if (response_id == GTK_RESPONSE_CLOSE)
-	{
-		g_object_unref (dialog);
-	}
-	else if (response_id == GTK_RESPONSE_HELP)
+	if (response == GTK_RESPONSE_HELP)
 	{
 		prefs_dialog_show_help (dialog);
+		return;
 	}
+		
+	g_object_unref (dialog);
 }
 
-void
+static void
 prefs_clear_cache_button_clicked_cb (GtkWidget *button,
-				     gpointer data)
+				     PrefsDialog *dialog)
 {
 	EphyEmbedShell *shell;
 	EphyEmbedSingle *single;
@@ -1276,7 +1482,7 @@ set_homepage_entry (EphyDialog *dialog,
 	g_value_unset (&value);
 }
 
-void
+static void
 prefs_homepage_current_button_clicked_cb (GtkWidget *button,
 					  EphyDialog *dialog)
 {
@@ -1296,9 +1502,93 @@ prefs_homepage_current_button_clicked_cb (GtkWidget *button,
 	set_homepage_entry (dialog, ephy_tab_get_address (tab));
 }
 
-void
+static void
 prefs_homepage_blank_button_clicked_cb (GtkWidget *button,
 					EphyDialog *dialog)
 {
 	set_homepage_entry (dialog, NULL);
 }
+static void
+prefs_dialog_init (PrefsDialog *pd)
+{
+	EphyDialog *dialog = EPHY_DIALOG (pd);
+	EphyEncodings *encodings;
+	GtkWidget *window, *curr_button, *blank_button;
+	GtkWidget *clear_cache_button, *font_prefs_button;
+#ifdef HAVE_GECKO_1_8
+	GtkWidget *css_checkbox, *css_edit_box, *css_edit_button, *css_container;
+#endif
+	gboolean sensitive;
+
+	pd->priv = EPHY_PREFS_DIALOG_GET_PRIVATE (pd);
+
+	ephy_dialog_construct (dialog,
+			       properties,
+			       ephy_file ("prefs-dialog.glade"),
+			       "prefs_dialog",
+			       NULL);
+
+	ephy_dialog_add_enum (dialog, properties[ACCEPT_COOKIES_PROP].id,
+			      G_N_ELEMENTS (cookies_accept_enum), cookies_accept_enum);
+
+	ephy_dialog_get_controls
+		(dialog,
+		 properties[WINDOW_PROP].id, &window,
+		 properties[HOMEPAGE_CURRENT_PROP].id, &curr_button,
+		 properties[HOMEPAGE_BLANK_PROP].id, &blank_button,
+		 properties[FONT_PREFS_BUTTON_PROP].id, &font_prefs_button,
+#ifdef HAVE_GECKO_1_8
+		 properties[CSS_CHECKBOX_PROP].id, &css_checkbox,
+		 properties[CSS_EDIT_BOX_PROP].id, &css_edit_box,
+		 properties[CSS_EDIT_BUTTON_PROP].id, &css_edit_button,
+		 properties[CSS_CONTAINER_PROP].id, &css_container,
+#endif
+		 properties[CLEAR_CACHE_BUTTON_PROP].id, &clear_cache_button,
+		 NULL);
+
+	ephy_gui_ensure_window_group (GTK_WINDOW (window));
+
+	g_signal_connect (window, "response",
+			  G_CALLBACK (prefs_dialog_response_cb), dialog);
+
+	g_signal_connect (curr_button, "clicked",
+			  G_CALLBACK (prefs_homepage_current_button_clicked_cb), dialog);
+	g_signal_connect (blank_button, "clicked",
+			  G_CALLBACK (prefs_homepage_blank_button_clicked_cb), dialog);
+
+	/* set homepage button sensitivity */
+	sensitive = eel_gconf_key_is_writable (CONF_GENERAL_HOMEPAGE);
+	gtk_widget_set_sensitive (curr_button, sensitive);
+	gtk_widget_set_sensitive (blank_button, sensitive);
+
+	g_signal_connect (font_prefs_button, "clicked",
+			  G_CALLBACK (font_prefs_button_clicked_cb), dialog);
+
+#ifdef HAVE_GECKO_1_8
+	css_checkbox_toggled (GTK_TOGGLE_BUTTON (css_checkbox), css_edit_box);
+	g_signal_connect (css_checkbox, "toggled",
+			  G_CALLBACK (css_checkbox_toggled), css_edit_box);
+
+	g_signal_connect (css_edit_button, "clicked",
+			  G_CALLBACK (css_edit_button_clicked_cb), dialog);
+
+	gtk_widget_show (css_container);
+#endif
+
+	g_signal_connect (clear_cache_button, "clicked",
+			  G_CALLBACK (prefs_clear_cache_button_clicked_cb), dialog);
+
+	encodings = EPHY_ENCODINGS (ephy_embed_shell_get_encodings
+					(EPHY_EMBED_SHELL (ephy_shell)));
+
+	create_node_combo (dialog, DEFAULT_ENCODING_PROP, encodings,
+			   ephy_encodings_get_all (encodings),
+			   CONF_LANGUAGE_DEFAULT_ENCODING, "ISO-8859-1");
+	create_autodetectors_combo (dialog, AUTO_ENCODING_PROP, encodings,
+				    ephy_encodings_get_detectors (encodings),
+				    CONF_LANGUAGE_AUTODETECT_ENCODING, "");
+
+	create_language_section	(dialog);
+
+	create_download_path_button (dialog);
+}
