@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: window-commands.c,v 1.187 2006/01/29 21:47:21 chpe Exp $
+ *  $Id: window-commands.c,v 1.189 2006/02/05 20:50:17 chpe Exp $
  */
 
 #include "config.h"
@@ -241,7 +241,8 @@ window_cmd_file_bookmark_page (GtkAction *action,
 	embed = ephy_window_get_active_embed (window);
 	g_return_if_fail (embed != NULL);
 
-	ephy_bookmarks_ui_add_bookmark (ephy_tab_get_address (tab),
+	ephy_bookmarks_ui_add_bookmark (GTK_WINDOW (window),
+					ephy_tab_get_address (tab),
 					ephy_tab_get_title (tab));
 }
 
@@ -743,8 +744,15 @@ window_cmd_help_about (GtkAction *action,
 		   "59 Temple Place, Suite 330, Boston, MA  02111-1307  USA")
 	};
 
-	char *licence;
+	EphyEmbedShell *shell;
+	EphyEmbedSingle *single;
+	char *licence, *comments;
 
+	shell = ephy_embed_shell_get_default ();
+	single = EPHY_EMBED_SINGLE (ephy_embed_shell_get_embed_single (shell));
+
+	comments = g_strdup_printf (_("Using “%s” backend"),
+				    ephy_embed_single_get_backend_name (single));
 	licence = g_strdup_printf ("%s\n\n%s\n\n%s",
 				   _(licence_part[0]), _(licence_part[1]), _(licence_part[2]));
 
@@ -754,6 +762,7 @@ window_cmd_help_about (GtkAction *action,
 			       "copyright", "Copyright © 2002-2004 Marco Pesenti Gritti\n"
 					    "Copyright © 2003-2006 The GNOME Web Browser Developers",
 			       "authors", authors,
+			       "comments", comments,
 			       "documenters", documenters,
 				/* Translators: This is a special message that shouldn't be translated
 				 * literally. It is used in the about box to give credits to
@@ -771,6 +780,7 @@ window_cmd_help_about (GtkAction *action,
 			       "wrap-license", TRUE,
 			       NULL);
 
+	g_free (comments);
 	g_free (licence);
 }
 
